<?php

/*****************************************************************************/
/* Calendar.php                                                              */
/*****************************************************************************/
/* Author: Aaron O'Neil - aaron@mud-master.com                               */
/* Version: 1.4                                                              */
/*****************************************************************************/
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* The GNU GPL can be found in gpl.txt in this directory                     */
/*****************************************************************************/

function ShowCalendar()
{
	global $days, $months, $months_short, $scripturl, $db_prefix, $imagesdir, $settings, $modSettings;
	global $ID_MEMBER, $txt, $yytitle, $mbname;

	// Passed as parameters to display a specific month.
	global $month, $year;

	$yytitle = "$mbname : $txt[calendar24]";
	template_header();

	$today = localtime(CalendarGetCurrentTime());
	// If the month and year are not passed in, using today's date as a starting point.
	if (!isset($month) || !isset($year))
	{
		$curMonth = $today;
		$month = $curMonth[4];
		$year = $curMonth[5]+1900;
	}
	else
	{
		if ($month < 0 || $month > 11)
			fatal_error($txt['calendar1']);
		if ($year < $modSettings['cal_minyear'] || $year > $modSettings['cal_maxyear'])
			fatal_error($txt['calendar2']);

		$curMonth = localtime(mktime(0,0,0,$month+1,1,$year));
	}

	// So I don't have to keep adding one to it all over.
	$monthp1 = $month+1;

	// Creating a date based on the first day of the current month. This is needed to figure out what day on
	// the first line of the calendar is used to start printing.
	$first = localtime(mktime(0,0,0,$monthp1,1,$year));

	// Find the last day of the month.
	$nLastDay = 31;
	while(!checkdate($monthp1,$nLastDay,$year) && $nLastDay > 0)
		$nLastDay--;

	// Just because I don't want to keep typing the array index. This is the number of days the first row
	// is shifted to the right for the starting day.
	$nShift = $first[6];

	if ($modSettings['cal_startmonday'] == 1)
	{
		$nShift--;
		if ($nShift<0)
			$nShift = 6;
	}

	// Number of rows required to fit the month.
	$nRows = floor(($nLastDay+$nShift)/7);
	if (($nLastDay+$nShift) % 7)
		$nRows++;

	// No point in doing this if over and over for access checks.
	if ($settings[7] == "Administrator" || $settings[7] == "Global Moderator")
		$bPowerUser = TRUE;
	else
		$bPowerUser = FALSE;

	// Creating an array of all the board categories with an array of all the member groups allowed for each category.
	// Using this to break it apart one time and use the ids from calendar table to look up the values.
	$cats = array();
	$rs = mysql_query("Select id_cat,membergroups From {$db_prefix}categories");
	while($row = mysql_fetch_array($rs))
		$cats[$row[0]] = explode(",",$row[1]);

	$bday = CalendarCreateBirthdayArray($monthp1,$year);
	$events = CalendarCreateEventArray($bPowerUser,$cats,$month,$year);
	$holidays = CalendarCreateHolidayArray($monthp1,$year);

	// Save some processing time by only doing this once. Particularly if the "Days as Link" option is
	// turned on.
	$bCanPost = CalendarCanPost();

	print "<table border=\"0\" width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" class=\"bordercolor\">";
	print "<tr><td>";

	print "<table cellspacing=\"1\" cellpadding=\"0\" width=\"100%\">";
	print "<caption><font size=\"5\" color=\"#$modSettings[cal_captioncolor]\">$months[$month] $year</font></caption>";

	$bStartMonday = ($modSettings['cal_startmonday'] == 1);
	$bShowWeek = ($modSettings['cal_showweeknum'] == 1);

	// Print the days of the week taking into consideration that they may want it to start on a monday.
	print "<tr>";
	for ($i=0;$i<7;$i++)
	{
		if ($bStartMonday)
		{
			$nDayIndex = $i+1;
			if ($nDayIndex == 7)
				$nDayIndex = 0;
		}
		else
			$nDayIndex = $i;
		print "<td class=\"windowbg\" width=\"14%\" align=\"center\">$days[$nDayIndex]</td>";
	}
	print "</tr>";

	// And adjustment value to apply to all calculated week numbers.
	if ($bShowWeek)
	{
		// Need to know what day the first of the year was on.
		$foy = localtime(mktime(0,0,0,1,1,$year));

		// If the first day of the year is on the start day of a week, then there is no adjustment
		// to be made. However, if the first day of the year is not a start day, then there is a partial
		// week at the start of the year that needs to be accounted for.
		if ($bStartMonday)
		{
			if ($foy[6]==1) // Monday
				$nWeekAdjust = 0;
			else
				$nWeekAdjust = 1;
		}
		else
		{
			if ($foy[6]==0) // Sunday
				$nWeekAdjust = 0;
			else
				$nWeekAdjust = 1;
		}
	}

	for ($nRow=0;$nRow<$nRows;$nRow++)
	{
		print "<tr>";
		for ($nCol=0;$nCol<7;$nCol++)
		{
			print "<td class=\"windowbg\" valign=\"top\" height=\"100\">";
			print "<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">";
			print "<tr valign=\"top\"><td>";

			$nDay = ($nRow*7)+$nCol-$nShift+1;
			if ($nDay >= 1 && $nDay <= $nLastDay)
			{
				// Showing week numbers?
				if (!$bShowWeek)
					$strWeek = "";
				else
				{
					// Is this the beginning of a week? Only displaying the week number on the first day of the week.
					if ((!$bStartMonday && ($first[6]+$nDay-1)%7 == 0) || ($bStartMonday && ($first[6]+$nDay-1)%7 == 1))
					{
						$nWeekNum = floor(($first[7]+$nDay-1)/7)+1+$nWeekAdjust;

						// I was looking at a calendar on the web and the last days of december if they do not fall
						// within 52 weeks are apparently called week 1; which sorta makes sense I suppose.
						if ($nWeekNum == 53)
							$nWeekNum = 1;

						$strWeek = "<font size=\"1\"> - $txt[calendar51] $nWeekNum</font>";
					}
					else
						$strWeek = "";
				}

				if ($today[5] == $curMonth[5] && $today[4] == $curMonth[4] && $nDay == $today[3])
				{
					print "<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%\"><tr>";
					print "<td bgcolor=\"#$modSettings[cal_todaycolor]\" valign=\"top\" height=\"100\" style=\"border:2px;border-style:outset\">";
					$PrintCurrentDay = true;
					CalendarPrintDay($bCanPost,$nDay,$month,$year,$strWeek);
				}
				else
					CalendarPrintDay($bCanPost,$nDay,$month,$year,$strWeek);

				if (isset($holidays[$nDay]))
					print ("<br><FONT size=\"1\" color=#$modSettings[cal_holidaycolor]>$holidays[$nDay]</font>");

				if (isset($bday[$nDay]))
				{
					// If there was a holiday, make a space.
					if (isset($holidays[$nDay]))
						print "<br>";
					print ("<br><FONT size=\"1\">$bday[$nDay]</font>");
				}

				if (isset($events[$nDay]))
				{
					// If there were birthdays or holidays, make a space.
					if (isset($bday[$nDay]) || isset($holidays[$nDay]))
						print "<br>";
					print ("<br><FONT size=\"1\">$events[$nDay]</font>");
				}

				if ($PrintCurrentDay)
				{
					$PrintCurrentDay = false;
					print "</td></tr></table>";
				}
			}

			print "</td></tr>";
			print "</table>";
			print "</td>";
		}

		print "</tr>";
	}

	print "<tr class=\"windowbg2\">";
	print "<form action=\"index.php?action=calendar\" method=\"post\">";
	
	print "<td>";
	if ($curMonth[4] > 0 || ($curMonth[4] == 0 && $curMonth[5]+1900 > $modSettings['cal_minyear']))
	{
		if ($curMonth[4] == 0)
		{
			$nPrevMonth = 11;
			$nPrevYear = $curMonth[5]-1+1900;
		}
		else
		{
			$nPrevMonth = $curMonth[4]-1;
			$nPrevYear = $curMonth[5]+1900;
		}
		print "<a href=\"$scripturl?action=calendar;year=$nPrevYear;month=$nPrevMonth\">&#160;&#171; $months_short[$nPrevMonth] $nPrevYear</a>";
	}
	print "</td>";

	$calPostGroups = explode(",",$modSettings['cal_postgroups']);

	print "<td align=\"center\">";
	if ($bCanPost)
		print "<a href=\"$scripturl?action=calendarpe;month=$month;year=$year\"><img src=\"$imagesdir/calendarpe.gif\" alt=\"$txt[calendar23]\" border=\"0\"></a>";
	print "</td>";

	print "<td colspan=3 align=\"center\">";
		print "<table border=\"0\" cellpadding=\"2\"><tr><td>";
		
		print "<select name=\"month\">";
		$nMonth = 0;
		foreach ($months as $strMonth)
		{
			if ($nMonth == $curMonth[4])
				print "<option value=$nMonth SELECTED>$strMonth</option>";
			else
				print "<option value=$nMonth>$strMonth</option>";
			$nMonth++;
		}
		
		print "</select>";
		print "&#160;<select name=\"year\">";
		for ($i=$modSettings['cal_minyear'];$i<=$modSettings['cal_maxyear'];$i++)
			if ($i == $curMonth[5]+1900)
				print "<option value=$i SELECTED>$i</option>";
			else
				print "<option value=$i>$i</option>";
		print "</select>";

		print "&#160;&#160;&#160;<input type=\"SUBMIT\" value=\"$txt[305]\">";

		print "</td></tr></table>";
	print "</td>";

	print "<td align=\"center\">";
	if ($bCanPost)
		print "<a href=\"$scripturl?action=calendarpe;month=$month;year=$year\"><img src=\"$imagesdir/calendarpe.gif\" alt=\"$txt[calendar23]\" border=\"0\"></a>";
	print "</td>";
	
	print "<td align=\"right\">";
	if ($curMonth[4] < 11 || ($curMonth[4] == 11 && $curMonth[5]+1900 < $modSettings['cal_maxyear']))
	{
		if ($curMonth[4] == 11)
		{
			$nNextMonth = 0;
			$nNextYear = $curMonth[5]+1+1900;
		}
		else
		{
			$nNextMonth = $curMonth[4]+1;
			$nNextYear = $curMonth[5]+1900;
		}
		print "<a href=\"$scripturl?action=calendar;year=$nNextYear;month=$nNextMonth\">$months_short[$nNextMonth] $nNextYear &#187;&#160;</a>";
	}
	print "</td>";
	
	print "</form>";
	print "</tr>";


	print "</td></tr></table>";
	print "</table>";

	footer();
	obExit();
}

// $day has been added as an optional parameter to restrict the results to a particular day.
// This is used by the board index to only find members of the current day.
function CalendarCreateBirthdayArray($monthp1,$year,$day=NULL)
{
	global $db_prefix, $scripturl, $txt, $modSettings;

	// Collect all of the birthdays for this month and precreate the strings to use for display.
	$bday = array();
	$strSql = "Select dayofmonth(birthdate) as dom,membername,realname,year(birthdate) From {$db_prefix}members Where month(birthdate) = $monthp1";
	if ($day != NULL)
		$strSql .= " And dayofmonth(birthdate) = $day";
	$rs = mysql_query($strSql);
	while($row = mysql_fetch_array($rs))
	{
		$euser=urlencode($row[1]);
		if ($row[3] > 0 && $row[3] <= $year)
		{
			$ageNum = $year-$row[3];
			$age = " ($ageNum)";
		}
		else
			$age = "";
		if (!isset($bday[$row[0]]))
			$bday[$row[0]] = "<font color=#$modSettings[cal_bdaycolor]>$txt[calendar3]</font> <a href=\"$scripturl?action=viewprofile;user=$euser\">$row[2]$age</a>";
		else
			$bday[$row[0]] .= ", <a href=\"$scripturl?action=viewprofile;user=$euser\">$row[2]$age</a>";
	}

	return($bday);
}

function CalendarCreateEventArray($bPowerUser,$cats,$month,$year,$day=NULL)
{
	global $db_prefix, $ID_MEMBER, $settings, $txt, $scripturl, $modSettings;

	$events = array();
	$strSql = "Select cal.day,cal.title,cal.id_board,b.id_cat,cal.id_topic,cal.id_member,cal.id ";
	$strSql .= "From {$db_prefix}calendar as cal, {$db_prefix}boards as b ";
	$strSql .= "Where cal.month=$month And cal.year=$year And cal.id_board = b.id_board";
	if ($day != NULL)
		$strSql .= " And cal.day = $day";
	$rs = mysql_query($strSql);
	while($row = mysql_fetch_assoc($rs))
	{
		if ($bPowerUser || $row['id_member'] == $ID_MEMBER)
			$strOwner = "<a href=\"$scripturl?action=calendaree;eventid=$row[id]\"><font color=#FF0000>*</font></a>";
		else
			$strOwner = "";

		// See if the user has access to the board this event was posted in.
		if ($cats[$row['id_cat']][0] == "" || $bPowerUser || in_array($settings[7],$cats[$row['id_cat']]))
		{
			if (!isset($events[$row['day']]))
				$events[$row['day']] = "<font color=#$modSettings[cal_eventcolor]>$txt[calendar4]</font> $strOwner<a href=\"$scripturl?board=$row[id_board];action=display;threadid=$row[id_topic]\">$row[title]</a>";
			else
				$events[$row['day']] .= ", $strOwner<a href=\"$scripturl?board=$row[id_board];action=display;threadid=$row[id_topic]\">$row[title]</a>";
		}
	}

	return($events);
}

// Builds an array of holiday strings for a particular month.
// Note that the month value passed in is one greater than the month value passed
// in to the display function.
function CalendarCreateHolidayArray($monthp1,$year,$day=NULL)
{
	global $db_prefix, $txt;

	// Holidays. Note that with the holidays the month in the table is NOT zero based like it is with
	// events. I did this because this table is more likely to get interaction directly from an admin
	// and having to remember to subtract one from the months is a pain.
	$holidays = array();
	$strSql = "Select day,year,title From {$db_prefix}calendar_holiday Where month=$monthp1 and (year Is Null or year=$year)";
	if ($day != NULL)
		$strSql .= " And day=$day";
	$rs = mysql_query($strSql);
	while($row = mysql_fetch_assoc($rs))
	{
		// The convention in the table is for holidays that always happen on the same date to have NULL for
		// the year value. If a row in the holiday table does have a year value then only print that holiday
		// for the specific year. Holidays that float can be added ahead of time with the year values given 
		// to make them show up on the right days.
		if ($row['year'] == NULL || $row['year'] == $year)
		{
			// I'm setting the colors on holidays as they print rather than in the string here. The reason
			// being I have holidays all one color and if multiple are on the same day it becomes akward
			// to try and set the <font> here -- or redundant if it happens on each one for each day.
			if (!isset($holidays[$row['day']]))
				$holidays[$row['day']] = "$txt[calendar5] $row[title]";
			else
				$holidays[$row['day']] .= ", $row[title]";
		}
	}

	return($holidays);
}

function CalendarPrintDay($bCanPost,$nDay,$month,$year,$strWeek)
{
	global $modSettings;

	if ($modSettings['cal_daysaslink'] == 1 && $bCanPost)
		print "<a href=\"$scripturl?action=calendarpe;month=$month;year=$year;day=$nDay\">$nDay</a>$strWeek";
	else
		print "$nDay$strWeek";	
}

function CalendarCanPost()
{
	global $settings, $modSettings, $ID_MEMBER, $username;

	if ($settings[7] == "Administrator" || $settings[7] == "Global Moderator")
		return(TRUE);
	
	if ($modSettings['cal_postgroups'] != "")
	{
		$calPostGroups = explode(",",$modSettings['cal_postgroups']);
		if (in_array($settings[7],$calPostGroups))
			return(TRUE);
	}

	if ($modSettings['cal_postmembers'] != "")
	{
		$calPostMems = explode(",",$modSettings['cal_postmembers']);
		if (in_array($username,$calPostMems))
			return(TRUE);
	}

	if ($modSettings['cal_memberscanpost'] == 1 && $ID_MEMBER > 0)
		return(TRUE);

	return(FALSE);
}

// Displays the form to start a calendar event.
function CalendarPostEvent()
{
	global $months, $settings, $db_prefix, $scripturl, $txt, $modSettings, $mbname, $yytitle;

	// Passed as parameters to display a specific month.
	global $month, $year, $day;

	// No point in doing this if over and over for access checks.
	if ($settings[7] == "Administrator" || $settings[7] == "Global Moderator")
		$bPowerUser = TRUE;
	else
		$bPowerUser = FALSE;

	$yytitle = "$mbname : $txt[calendar23]";
	template_header();

	$calPostGroups = explode(",",$modSettings['cal_postgroups']);
	if (!CalendarCanPost())
		fatal_error($txt['calendar6']);

	if (!isset($month))
		fatal_error($txt['calendar7']);
	if (!isset($year))
		fatal_error($txt['calendar8']);

	if ($month < 0 || $month > 11)
		fatal_error($txt['calendar1']);
	if ($year < $modSettings['cal_minyear'] || $year > $modSettings['cal_maxyear'])
		fatal_error($txt['calendar2']);

	// Find the last day of the month.
	$nLastDay = 31;
	while(!checkdate($month+1,$nLastDay,$year) && $nLastDay > 0)
		$nLastDay--;

	print "<center>";
	print "<form action=\"$scripturl?action=post\" method=\"post\">";
	print "<input type=\"hidden\" name=\"year\" value=\"$year\">";
	print "<input type=\"hidden\" name=\"month\" value=\"$month\">";
	print "<input type=\"hidden\" name=\"title\" value=\"$txt[464]\">";
	print "<input type=\"hidden\" name=\"linkcalendar\" value=\"1\">";

	print "<table border=\"0\">";

	print "<tr>";
	print "<td>$txt[calendar9]</td>";
	print "<td>$months[$month]</td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar10]</td>";
	print "<td>$year</td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar11]</td>";
	print "<td>";
	print "<select name=\"day\">";
	for ($i=1;$i<=$nLastDay;$i++)
		if (isset($day) && $i == $day)
			print "<option value=\"$i\" selected>$i</option>";
		else
			print "<option value=\"$i\">$i</option>";
	print "</select>";
	print "</td>";
	print "</tr>";

	// Show the number of days to span only if enabled and set for more than one day.
	if ($modSettings['cal_allowspan'] == 1 && $modSettings['cal_maxspan'] > 1)
	{
		print "<tr>";
		print "<td>$txt[calendar54]</td>";
		print "<td>";
		print "<select name=\"span\">";
		for ($i=1;$i<=$modSettings['cal_maxspan'];$i++)
			if ($i == 1)
				print "<option value=\"$i\" selected>$i</option>";
			else
				print "<option value=\"$i\">$i</option>";
		print "</select>";
		print "</td>";
		print "</tr>";
	}

	print "<tr>";
	print "<td>$txt[calendar12]</td>";
	print "<td><input name=\"evtitle\" type=\"text\" maxlength=\"30\" size=\"30\"></td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar13]</td>";
	print "<td>";
	print "<select name=\"board\">";
	$rs = mysql_query("select b.id_board,b.name,c.memberGroups,c.name From {$db_prefix}boards as b, {$db_prefix}categories as c Where b.id_cat = c.id_cat");
	while($row = mysql_fetch_array($rs))
	{
		$groups = explode(",",$row[2]);

		if ($row[2] == "" || $bPowerUser || in_array($settings[7],$groups))
		{
			$board = "$row[3] - $row[1]";
			if ($board == $modSettings['cal_defaultboard'])
				print "<option value=\"$row[0]\" selected>$board</option>";
			else
				print "<option value=\"$row[0]\">$board</option>";
		}
	}
	print "</select>";
	print "</td>";
	print "</tr>";

	print "<tr><td align=\"center\" colspan=\"2\"><input type=\"submit\" value=\"$txt[calendar23]\"></td></tr>";

	print "</table>";
	print "</form>";
	print "</center>";

	footer();
	obExit();
}

// Called from the posting routine to make sure all of the calendar elements exist
// and are valid.
function CalendarValidatePost()
{
	global $modSettings, $txt;

	// Passed in from form.
	global $month, $year, $day, $evtitle, $deleteevent, $span;

	if (!isset($month))
		fatal_error($txt['calendar7']);
	if (!isset($year))
		fatal_error($txt['calendar8']);

	if ($month < 0 || $month > 11)
		fatal_error($txt['calendar1']);
	if ($year < $modSettings['cal_minyear'] || $year > $modSettings['cal_maxyear'])
		fatal_error($txt['calendar2']);

	if (!CalendarCanPost())
		fatal_error($txt['calendar6']);

	if (isset($span))
	{
		// Make sure it's turned on and not some fool trying to trick it.
		if ($modSettings['cal_allowspan'] != 1)
			fatal_error($txt['calendar55']);
		if ($span < 1 || $span > $modSettings['cal_maxspan'])
			fatal_error($txt['calendar56']);
	}

	// Started using this function in some other places. There is no need to validate
	// the following values if we are just deleting the event.
	if (!isset($deleteevent))
	{
		if (!isset($day))
			fatal_error($txt['calendar14']);
		if (!isset($evtitle))
			fatal_error($txt['calendar15']);

		if (!checkdate($month+1,$day,$year))
			fatal_error($txt['calendar16']);

		if (trim($evtitle)=="")
			fatal_error($txt['calendar17']);
		if (strlen($evtitle) > 30) 
			$evtitle = substr($evtitle,0,30);
		$evtitle = str_replace(";","",$evtitle);
	}
}

function CalendarEditEvent()
{
	global $ID_MEMBER, $modSettings, $txt, $mbname, $yytitle;
	global $months, $settings, $db_prefix, $scripturl;
	
	// Passed as web param.
	global $eventid;

	if (!isset($eventid))
		fatal_error($txt['calendar18']);

	if ($settings[7] == "Administrator" || $settings[7] == "Global Moderator")
		$bPowerUser = TRUE;
	else
		$bPowerUser = FALSE;

	$rs = mysql_query("Select title,month,day,year,id_member From {$db_prefix}calendar Where id='$eventid'");
	$row = mysql_fetch_assoc($rs);

	if (!$bPowerUser)
		if ($row['id_member'] != $ID_MEMBER)
			fatal_error($txt['calendar19']);

	$yytitle = "$mbname : $txt[calendar20]";
	template_header();

	print "<script language = \"JavaScript\">";
	print "function DoConfirm(message, url) { if(confirm(message)) location.href = url; }";
	print "</script>";

	print "<center>";
	print "<form action=\"$scripturl?action=calendaree2\" method=\"post\">";
	print "<input type=\"hidden\" name=\"eventid\" value=\"$eventid\">";

	print "<table border=\"0\">";

	print "<tr>";
	print "<td>$txt[calendar9]</td>";
	print "<td>";
	print "<select name=\"month\">";
	$nMonth = 0;
	foreach ($months as $m)
	{
		if ($nMonth == $row['month'])
			print "<option value=\"$nMonth\" SELECTED>$m</option>";
		else
			print "<option value=\"$nMonth\">$m</option>";
		$nMonth++;
	}
	print "</select>";
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar10]</td>";
	print "<td>";
	print "<select name=\"year\">";
	for ($i=$modSettings['cal_minyear'];$i<=$modSettings['cal_maxyear'];$i++)
		if ($i == $row['year'])
			print "<option value=$i SELECTED>$i</option>";
		else
			print "<option value=$i>$i</option>";
	print "</select>";
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar11]</td>";
	print "<td>";
	print "<select name=\"day\">";
	// Since we don't know which month the user will choose, providing the max 31 days. The date will be
	// validated when they try to post the event.
	for ($i=1;$i<=31;$i++)
		if ($i == $row['day'])
			print "<option value=\"$i\" SELECTED>$i</option>";
		else
			print "<option value=\"$i\">$i</option>";
	print "</select>";
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar12]</td>";
	print "<td><input name=\"evtitle\" type=\"text\" maxlength=\"30\" size=\"30\" value=\"$row[title]\"></td>";
	print "</tr>";

	print "<tr>";
	print "<td align=\"center\" colspan=\"2\">";
	print "<input type=\"submit\" value=\"$txt[calendar20]\">";
	print "&#160;&#160;&#160";
	print "<a href=\"javascript:DoConfirm('$txt[calendar21]','$scripturl?action=calendaree2;deleteevent=1;month=$row[month];year=$row[year];eventid=$eventid');\">$txt[calendar22]</a>";
	print "</td>";
	print "</tr>";

	print "</table>";
	print "</form>";
	print "</center>";

	footer();
	obExit();
}

function CalendarEditEvent2()
{
	global $settings, $db_prefix, $scripturl, $yySetLocation;
	global $ID_MEMBER;
	
	// Passed as web param.
	global $eventid, $month, $day, $year, $evtitle, $deleteevent;

	if (!isset($eventid))
		fatal_error($txt['calendar18']);

	CalendarValidatePost();

	if ($settings[7] != "Administrator" && $settings[7] != "Global Moderator")
	{
		$rs = mysql_query("Select id_member From {$db_prefix}calendar Where id='$eventid'");
		$row = mysql_fetch_assoc($rs);

		if ($row['id_member'] != $ID_MEMBER)
			fatal_error($txt['calendar19']);
	}

	if (get_magic_quotes_gpc()==0)
		$evtitle = mysql_escape_string($evtitle);
	$evtitle = htmlspecialchars($evtitle);

	if (isset($deleteevent))
		mysql_query("Delete From {$db_prefix}calendar Where id='$eventid'");
	else		
		mysql_query("Update {$db_prefix}calendar Set month='$month',day='$day',year='$year',title='$evtitle' Where id='$eventid'");

	$yySetLocation = "$scripturl?action=calendar;month=$month;year=$year";
	redirectexit();
}

// Called to add the hidden form elements to carry the data around while posting/previewing.
function CalendarAddHidden()
{
	global $year, $month, $day, $evtitle, $linkcalendar, $span;

	print "<input type=\"hidden\" name=\"year\" value=\"$year\">";
	print "<input type=\"hidden\" name=\"month\" value=\"$month\">";
	print "<input type=\"hidden\" name=\"day\" value=\"$day\">";
	print "<input type=\"hidden\" name=\"evtitle\" value=\"$evtitle\">";
	print "<input type=\"hidden\" name=\"linkcalendar\" value=\"1\">";
	
	if (isset($span))
		print "<input type=\"hidden\" name=\"span\" value=\"$span\">";
}

// Returns TRUE if this user is allowed to link the topic in question.
function CalendarCanLink()
{
	global $ID_MEMBER, $txt, $settings, $db_prefix, $username;
	global $threadid, $board;

	if (!CalendarCanPost())
		fatal_error($txt['calendar6']);

	if (!isset($board))
		fatal_error($txt['calendar38']);
	if (!isset($threadid))
		fatal_error($txt['calendar39']);

	// If not an admin or global mod make sure they are either the owner of the topic they are trying to
	// link or a moderator of that board.
	if ($settings[7] != "Administrator" && $settings[7] != "Global Moderator")
	{
		$rs = mysql_query("Select moderators from {$db_prefix}boards where id_board='$board'");
		if ($row = mysql_fetch_row($rs))
		{
			$mods = explode(",",$row[0]);
			if (in_array($username,$mods))
				return(TRUE);

			// Not admin, global mod, or a moderator of this board. Only thing left is to see if they are the
			// owner of the topic.
			$rs = mysql_query("Select id_member_started from {$db_prefix}topics where id_topic='$threadid'");
			if ($row = mysql_fetch_row($rs))
			{
				if ($row[0] != $ID_MEMBER)
					fatal_error($txt['calendar41']); // Not the owner of the topic.
			}
			else
				fatal_error($txt['calendar40']);	// Topic doesn't exist.
		}
		else
			fatal_error($txt['calendar42']);	// Board doesn't exist.
	}

	return(TRUE);
}

function CalendarLinkEvent()
{
	global $ID_MEMBER, $modSettings, $txt, $mbname, $yytitle, $months;
	global $settings, $db_prefix, $scripturl, $threadid, $board;

	CalendarCanLink();

	$yytitle = "$mbname : $txt[calendar43]";
	template_header();

	$today = localtime(CalendarGetCurrentTime());
	$day = $today[3];
	$month = $today[4];
	$year = $today[5]+1900;

	print "<center>";
	print "<form action=\"$scripturl?action=calendarle2\" method=\"post\">";
	print "<input type=\"hidden\" name=\"board\" value=\"$board\">";
	print "<input type=\"hidden\" name=\"threadid\" value=\"$threadid\">";

	print "<table border=\"0\">";

	print "<tr>";
	print "<td>$txt[calendar9]</td>";
	print "<td>";
	print "<select name=\"month\">";
	$nMonth = 0;
	foreach ($months as $m)
	{
		if ($nMonth == $month)
			print "<option value=\"$nMonth\" selected>$m</option>";
		else
			print "<option value=\"$nMonth\">$m</option>";
		$nMonth++;
	}
	print "</select>";
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar10]</td>";
	print "<td>";
	print "<select name=\"year\">";
	for ($i=$modSettings['cal_minyear'];$i<=$modSettings['cal_maxyear'];$i++)
		if ($year == $i)
			print "<option value=$i selected>$i</option>";
		else
			print "<option value=$i>$i</option>";
	print "</select>";
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td>$txt[calendar11]</td>";
	print "<td>";
	print "<select name=\"day\">";
	// Since we don't know which month the user will choose, providing the max 31 days. The date will be
	// validated when they try to post the event.
	for ($i=1;$i<=31;$i++)
		if ($day == $i)
			print "<option value=\"$i\" selected>$i</option>";
		else
			print "<option value=\"$i\">$i</option>";
	print "</select>";
	print "</td>";
	print "</tr>";

	// Show the number of days to span only if enabled and set for more than one day.
	if ($modSettings['cal_allowspan'] == 1 && $modSettings['cal_maxspan'] > 1)
	{
		print "<tr>";
		print "<td>$txt[calendar54]</td>";
		print "<td>";
		print "<select name=\"span\">";
		for ($i=1;$i<=$modSettings['cal_maxspan'];$i++)
			if ($i == 1)
				print "<option value=\"$i\" selected>$i</option>";
			else
				print "<option value=\"$i\">$i</option>";
		print "</select>";
		print "</td>";
		print "</tr>";
	}

	print "<tr>";
	print "<td>$txt[calendar12]</td>";
	print "<td><input name=\"evtitle\" type=\"text\" maxlength=\"30\" size=\"30\"></td>";
	print "</tr>";

	print "<tr>";
	print "<td align=\"center\" colspan=\"2\">";
	print "<input type=\"submit\" value=\"$txt[calendar43]\">";
	print "</td>";
	print "</tr>";

	print "</table>";
	print "</form>";
	print "</center>";

	footer();
	obExit();
}

function CalendarLinkEvent2()
{
	global $scripturl, $yySetLocation, $ID_MEMBER;
	global $board, $threadid, $evtitle, $month, $day, $year, $span;

	CalendarCanLink();
	CalendarValidatePost();

	CalendarInsertEvent($board,$threadid,$evtitle,$ID_MEMBER,$month,$day,$year,$span);

	$yySetLocation = "$scripturl?action=calendar;month=$month;year=$year";
	redirectexit();
}

// Consolidating the various insert statements into this function.
function CalendarInsertEvent($id_board,$id_topic,$title,$id_member,$month,$day,$year,$span)
{
	global $db_prefix;

	if (get_magic_quotes_gpc()==0)
		$title = mysql_escape_string($title);
	$title = htmlspecialchars($title);

	if ($span == NULL || trim($span) == "")
		mysql_query("Insert Into {$db_prefix}calendar(id_board,id_topic,title,id_member,month,day,year) values('$id_board','$id_topic','$title','$id_member','$month','$day','$year')");
	else
	{
		// I went for the simplest way I could think of for making the events span multiple days. For each day
		// a row is added to the calendar table.
		$tVal = mktime(0,0,0,$month+1,$day,$year);

		for ($i=0;$i<$span;$i++)
		{
			$eventTime = localtime($tVal);
			$year = $eventTime[5]+1900;
			mysql_query("Insert Into {$db_prefix}calendar(id_board,id_topic,title,id_member,month,day,year) values('$id_board','$id_topic','$title','$id_member',$eventTime[4],$eventTime[3],$year)");
			$tVal = strtotime("tomorrow",$tVal);
		}
	}
}

function CalendarGetCurrentTime()
{
	global $timeoffset, $settings;

	$timeUserOff = (isset($settings[18]) ? $settings[18] : 0);
	return(time()+(($timeoffset+$timeUserOff)*3600));
}

// Called from BoardIndex.php to display the current day's events on the board index.
function CalendarDoIndex()
{
	global $modSettings, $db_prefix, $txt, $imagesdir, $settings, $scripturl;

	if ($modSettings['cal_enabled'] != 1)
		return;

	// Make sure at least one of the options is checked.
	if ($modSettings['cal_showeventsonindex'] != 1 && $modSettings['cal_showbdaysonindex'] != 1 && $modSettings['cal_showholidaysonindex'] != 1)
		return;

	$today = localtime(CalendarGetCurrentTime());
	$month = $today[4];
	$year = $today[5]+1900;
	$day = $today[3];

	$header =  "<tr><td class=\"catbg\" colspan=\"2\"><b>$txt[calendar47]</b></td></tr>";
	$header .= "<tr><td class=\"windowbg\" width=\"20\" valign=\"middle\" align=\"center\"><a href=\"$scripturl?action=calendar\"><img src=\"$imagesdir/calindex.gif\" border=\"0\" width=\"20\" alt=\"$txt[calendar24]\"></a></td>";
	$header .= "<td class=\"windowbg2\" width=\"100%\">";

	$bPrintedHeader = FALSE;

	if ($modSettings['cal_showholidaysonindex'] == 1)
	{
		$holidays = CalendarCreateHolidayArray($month+1,$year,$day);
		if (isset($holidays[$day]))
		{
			if (!$bPrintedHeader)
			{
				print $header;
				$bPrintedHeader = TRUE;
			}
			print "<FONT size=\"1\" color=#$modSettings[cal_holidaycolor]>$holidays[$day]</font>";
		}
	}

	if ($modSettings['cal_showbdaysonindex'] == 1)
	{
		$bday = CalendarCreateBirthdayArray($month+1,$year,$day);
		if (isset($bday[$day]))
		{
			if (!$bPrintedHeader)
			{
				print $header;
				$bPrintedHeader = TRUE;
			}
			if (isset($holidays[$day]))
				print "<br>";
			print "<font size=\"1\">$bday[$day]</font>";
		}
	}

	if ($modSettings['cal_showeventsonindex'] == 1)
	{
		if ($settings[7] == "Administrator" || $settings[7] == "Global Moderator")
			$bPowerUser = TRUE;
		else
			$bPowerUser = FALSE;

		$cats = array();
		$rs = mysql_query("Select id_cat,membergroups From {$db_prefix}categories");
		while($row = mysql_fetch_array($rs))
			$cats[$row[0]] = explode(",",$row[1]);

		$events = CalendarCreateEventArray($bPowerUser,$cats,$month,$year,$day);
		if (isset($events[$day]))
		{
			if (!$bPrintedHeader)
			{
				print $header;
				$bPrintedHeader = TRUE;
			}
			if (isset($bday[$day]) || isset($holidays[$day]))
				print "<br>";
			print "<font size=\"1\">$events[$day]</font>";
		}
	}

	if ($bPrintedHeader)
		print "</td></tr>";
}
?>

